import os
import os.path
import fnmatch

import yaml

import mplayer

from alsa import *

class player(object):
    instance = None
    def __new__(cls):
        if cls.instance is None:
            cls.instance = mplayer.Player(args = ["-nolirc"])
        return cls.instance

class Cache:
    def __init__(self, driver, name, data = None):
        self._driver = driver
        self._name = name
        self._data = data
        
        self.cache_dir = "cache"
        self.file_dir = os.path.join("cache", self._driver)
        
        try:
            self.load()
        except IOError:
            self.write()
    
    def write(self):
        if not os.path.isdir(self.file_dir):
            os.makedirs(self.file_dir)
        
        f = open(os.path.join(self.file_dir, self._name + ".yml"), "w")
        yaml.dump(self._data, f)
        f.close()
    
    def load(self):
        f = open(os.path.join("cache", self._driver, self._name + ".yml"))
        self._data = yaml.load(f)
        f.close()

class History(Cache):
    def __init__(self, driver):
        Cache.__init__(self, driver, "history", data = [])
    
    def append(self, path):
        self._data.append(path)
        self.write()
    
    def update_position(self, position):
        if len(self._data) > 0:
            self._data[-1]['position'] = position
    
    def last(self, pattern):
        pattern = "*" + pattern + "*"
        
        match = []
        for x in self._data:
            if fnmatch.fnmatch(x['path'], pattern):
                match.append(x)
        
        if len(match) > 0:
            return match[-1]
        else:
            return None

class MPlayer(Alsa):
    def __init__(self, config):
        Alsa.__init__(self, config)
        self._config = config
        self._player = player()
        # self._player.args = ['-really-quiet', '-msglevel', 'all=-1']
        self._player.spawn()
        self._url = None
    
    def play(self, url, name = None, position = None):
        self._player.spawn()
        self._url = url
        self._player.loadfile("\"" + url + "\"")
        if name is None:
            self._osd.display(os.path.basename(url))
        else:
            self._osd.display(name)
        if position is not None:
            self.pause()
            self.position(position)
            self.pause()
        
        return url
    
    def pause(self):
        self._player.spawn()
        self._player.pause()
    
    def seek(self, step):
        self._player.spawn()
        step = float(step)
        self._player.time_pos = mplayer.Step(step)
    
    def stop(self):
        self._player.spawn()
        self._player.stop()
    
    def position(self, value = None):
        self._player.spawn()
        if value is None:
            return self._player.time_pos
        else:
            self._player.time_pos = float(value)

class MPlayerFilms(MPlayer):
    def __init__(self, config):
        MPlayer.__init__(self, config)
        self._stopped = False
        self._history = History(self.__class__.__name__)
    
    def play(self, path, **kwargs):
        if not self._stopped:
            pos = self.position()
            self._history.update_position(pos)
            self._history.write()
        
        if os.path.exists(path):
            MPlayer.play(self, path, **kwargs)
            self._history.append({'path': path, 'position': 0.})
            return path
    
    def stop(self):
        pos = self.position()
        self._history.update_position(pos)
        MPlayer.stop(self)
        self._history.write()
        self._stopped = True
    
    def continue_last(self, pattern):
        if not self._stopped:
            pos = self.position()
            self._history.update_position(pos)
            self._history.write()
        
        last = self._history.last(pattern)
        path = last['path']
        pos  = last['position']
        if pos is None:
            self._url = path
            self.next()
        else:
            self.play(path, position = pos)
    
    def nextprev(self, direction):
        if self._url is None:
            return
        
        dir = os.path.dirname(self._url)
        f = os.path.basename(self._url)
        
        files = os.listdir(dir)
        files.sort()
        n = len(files)
        
        new_f = f
        for i in xrange(n):
            if files[i] == f:
                if direction > 0:
                    new_f = files[(i + 1) % n]
                else:
                    new_f = files[(i - 1) % n]
                break
        
        self.play(os.path.join(dir, new_f))
        return self._url
    
    def next(self):
        return self.nextprev(+1)

    def prev(self):
        return self.nextprev(-1)

class MPlayerTV(MPlayer):
    def __init__(self, channels):
        self._channels = channels
        MPlayer.__init__(self, channels)
    
    def play(self, channel):
        if channel in self._channels:
            url = "dvb://" + self._channels[channel]
            return MPlayer.play(self, url)
        else:
            return None
    
    def position(self, value = None):
        pass

class MPlayerRadio(MPlayer):
    def __init__(self, channels):
        self._channels = channels
        MPlayer.__init__(self, channels)
    
    def play(self, channel):
        if channel in self._channels:
            url = self._channels[channel]
            return MPlayer.play(self, url, name = channel)
        else:
            return None
    
    def position(self, value = None):
        pass

